@@ -97,6 +97,66 @@ dead_code_constant_boolean_should_warn_more: {
     node_version: "<=4"
 }
 
+trim_try: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        try {
+            var a;
+        } catch (e) {
+            console.log("FAIL");
+        } finally {
+            console.log(a);
+        }
+    }
+    expect: {
+        var a;
+        console.log(a);
+    }
+    expect_stdout: "undefined"
+}
+
+trim_finally_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        try {
+            console.log("PASS");
+        } finally {
+            var a;
+        }
+    }
+    expect: {
+        console.log("PASS");
+        var a;
+    }
+    expect_stdout: "PASS"
+}
+
+trim_finally_2: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        try {
+            console.log("PASS");
+        } catch (e) {
+        } finally {
+            var a;
+        }
+    }
+    expect: {
+        try {
+            console.log("PASS");
+            var a;
+        } catch (e) {
+        }
+    }
+    expect_stdout: "PASS"
+}
+
 try_catch_finally: {
     options = {
         conditionals: true,
@@ -130,10 +190,7 @@ try_catch_finally: {
             a = 3;
             console.log("PASS");
         }();
-        try {
-            console.log(a);
-        } finally {
-        }
+        console.log(a);
     }
     expect_stdout: [
         "PASS",
@@ -141,207 +198,6 @@ try_catch_finally: {
     ]
 }
 
-accessor: {
-    options = {
-        side_effects: true,
-    }
-    input: {
-        ({
-            get a() {},
-            set a(v){
-                this.b = 2;
-            },
-            b: 1
-        });
-    }
-    expect: {}
-}
-
-issue_2233_1: {
-    options = {
-        pure_getters: "strict",
-        side_effects: true,
-        unsafe: true,
-    }
-    input: {
-        Array.isArray;
-        Boolean;
-        console.log;
-        Date;
-        decodeURI;
-        decodeURIComponent;
-        encodeURI;
-        encodeURIComponent;
-        Error.name;
-        escape;
-        eval;
-        EvalError;
-        Function.length;
-        isFinite;
-        isNaN;
-        JSON;
-        Math.random;
-        Number.isNaN;
-        parseFloat;
-        parseInt;
-        RegExp;
-        Object.defineProperty;
-        String.fromCharCode;
-        RangeError;
-        ReferenceError;
-        SyntaxError;
-        TypeError;
-        unescape;
-        URIError;
-    }
-    expect: {}
-    expect_stdout: true
-}
-
-global_timeout_and_interval_symbols: {
-    options = {
-        pure_getters: "strict",
-        side_effects: true,
-        unsafe: true,
-    }
-    input: {
-        // These global symbols do not exist in the test sandbox
-        // and must be tested separately.
-        clearInterval;
-        clearTimeout;
-        setInterval;
-        setTimeout;
-    }
-    expect: {}
-}
-
-issue_2233_2: {
-    options = {
-        pure_getters: "strict",
-        reduce_funcs: true,
-        reduce_vars: true,
-        side_effects: true,
-        unsafe: true,
-        unused: true,
-    }
-    input: {
-        var RegExp;
-        Array.isArray;
-        RegExp;
-        UndeclaredGlobal;
-        function foo() {
-            var Number;
-            AnotherUndeclaredGlobal;
-            Math.sin;
-            Number.isNaN;
-        }
-    }
-    expect: {
-        var RegExp;
-        UndeclaredGlobal;
-        function foo() {
-            var Number;
-            AnotherUndeclaredGlobal;
-            Number.isNaN;
-        }
-    }
-}
-
-issue_2233_3: {
-    options = {
-        pure_getters: "strict",
-        reduce_funcs: true,
-        reduce_vars: true,
-        side_effects: true,
-        toplevel: true,
-        unsafe: true,
-        unused: true,
-    }
-    input: {
-        var RegExp;
-        Array.isArray;
-        RegExp;
-        UndeclaredGlobal;
-        function foo() {
-            var Number;
-            AnotherUndeclaredGlobal;
-            Math.sin;
-            Number.isNaN;
-        }
-    }
-    expect: {
-        UndeclaredGlobal;
-    }
-}
-
-global_fns: {
-    options = {
-        side_effects: true,
-        unsafe: true,
-    }
-    input: {
-        Boolean(1, 2);
-        decodeURI(1, 2);
-        decodeURIComponent(1, 2);
-        Date(1, 2);
-        encodeURI(1, 2);
-        encodeURIComponent(1, 2);
-        Error(1, 2);
-        escape(1, 2);
-        EvalError(1, 2);
-        isFinite(1, 2);
-        isNaN(1, 2);
-        Number(1, 2);
-        Object(1, 2);
-        parseFloat(1, 2);
-        parseInt(1, 2);
-        RangeError(1, 2);
-        ReferenceError(1, 2);
-        String(1, 2);
-        SyntaxError(1, 2);
-        TypeError(1, 2);
-        unescape(1, 2);
-        URIError(1, 2);
-        try {
-            Function(1, 2);
-        } catch (e) {
-            console.log(e.name);
-        }
-        try {
-            RegExp(1, 2);
-        } catch (e) {
-            console.log(e.name);
-        }
-        try {
-            Array(NaN);
-        } catch (e) {
-            console.log(e.name);
-        }
-    }
-    expect: {
-        try {
-            Function(1, 2);
-        } catch (e) {
-            console.log(e.name);
-        }
-        try {
-            RegExp(1, 2);
-        } catch (e) {
-            console.log(e.name);
-        }
-        try {
-            Array(NaN);
-        } catch (e) {
-            console.log(e.name);
-        }
-    }
-    expect_stdout: [
-        "SyntaxError",
-        "SyntaxError",
-        "RangeError",
-    ]
-}
-
 collapse_vars_assignment: {
     options = {
         collapse_vars: true,
@@ -380,7 +236,7 @@ collapse_vars_lvalues_drop_assign: {
     }
 }
 
-collapse_vars_misc1: {
+collapse_vars_misc: {
     options = {
         collapse_vars: true,
         dead_code: true,
@@ -863,23 +719,6 @@ issue_2749: {
     expect_stdout: "PASS"
 }
 
-unsafe_builtin: {
-    options = {
-        side_effects: true,
-        unsafe: true,
-    }
-    input: {
-        (!w).constructor(x);
-        Math.abs(y);
-        [ 1, 2, z ].valueOf();
-    }
-    expect: {
-        w, x;
-        y;
-        z;
-    }
-}
-
 issue_2860_1: {
     options = {
         dead_code: true,
@@ -892,9 +731,7 @@ issue_2860_1: {
         }());
     }
     expect: {
-        console.log(function(a) {
-            return 1 ^ a;
-        }());
+        console.log(1);
     }
     expect_stdout: "1"
 }
@@ -943,24 +780,6 @@ issue_2929: {
     expect_stdout: "PASS"
 }
 
-unsafe_string_replace: {
-    options = {
-        side_effects: true,
-        unsafe: true,
-    }
-    input: {
-        "foo".replace("f", function() {
-            console.log("PASS");
-        });
-    }
-    expect: {
-        "foo".replace("f", function() {
-            console.log("PASS");
-        });
-    }
-    expect_stdout: "PASS"
-}
-
 issue_3402: {
     options = {
         dead_code: true,
@@ -1013,3 +832,533 @@ issue_3406: {
     }
     expect_stdout: "true"
 }
+
+function_assign: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        console.log(function() {
+            var a = "PASS";
+            function h(c) {
+                return c;
+            }
+            h.p = function(b) {
+                return b;
+            }.p = a;
+            return h;
+        }().p);
+    }
+    expect: {
+        console.log(function() {
+            var a = "PASS";
+            function h(c) {
+                return c;
+            }
+            h.p = a;
+            return h;
+        }().p);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3552: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+    }
+    input: {
+        var a = "PASS";
+        (function() {
+            (1..p += 42) && (a = "FAIL");
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "PASS";
+        (function() {
+            (1..p += 42) && (a = "FAIL");
+        })();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+unreachable_assign: {
+    options = {
+        dead_code: true,
+        strings: true,
+    }
+    input: {
+        console.log(A = "P" + (A = "A" + (B = "S" + (A = B = "S"))), A, B);
+    }
+    expect: {
+        console.log(A = "P" + "A" + (B = "S" + "S"), A, B);
+    }
+    expect_stdout: "PASS PASS SS"
+}
+
+catch_return_assign: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        console.log(function() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                return e = "PASS";
+            }
+        }());
+    }
+    expect: {
+        console.log(function() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                return "PASS";
+            }
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3578: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = "FAIL", b, c;
+        try {
+            b = c.p = b = 0;
+        } catch (e) {
+            b += 42;
+            b && (a = "PASS");
+        }
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL", b, c;
+        try {
+            b = c.p = b = 0;
+        } catch (e) {
+            b += 42;
+            b && (a = "PASS");
+        }
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var o = {
+            set p(v) {
+                o = o.p = o = v;
+            }
+        };
+        o.p = "PASS";
+        console.log(o);
+    }
+    expect: {
+        var o = {
+            set p(v) {
+                o = o.p = o = v;
+            }
+        };
+        o.p = "PASS";
+        console.log(o);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_2: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = o[a] = a = v;
+            }
+        };
+        o[a] = "PASS";
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = o[a] = a = v;
+            }
+        };
+        o[a] = "PASS";
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_3: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            return a;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = o[f()] = a = v;
+            }
+        };
+        o[f()] = "PASS";
+        console.log(a);
+    }
+    expect: {
+        function f() {
+            return a;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = o[f()] = a = v;
+            }
+        };
+        o[f()] = "PASS";
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_4: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            return o;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = f()[a] = a = v;
+            }
+        };
+        f()[a] = "PASS";
+        console.log(a);
+    }
+    expect: {
+        function f() {
+            return o;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = f()[a] = a = v;
+            }
+        };
+        f()[a] = "PASS";
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_5: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            return o;
+        }
+        function g() {
+            return a;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = f()[g()] = a = v;
+            }
+        };
+        f()[g()] = "PASS";
+        console.log(a);
+    }
+    expect: {
+        function f() {
+            return o;
+        }
+        function g() {
+            return a;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                a = f()[g()] = a = v;
+            }
+        };
+        f()[g()] = "PASS";
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3830_6: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            return o;
+        }
+        function g() {
+            return a;
+        }
+        function h(v) {
+            a = f()[g()] = a = v;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                h(v);
+            }
+        };
+        o.FAIL = "PASS";
+        console.log(a);
+    }
+    expect: {
+        function f() {
+            return o;
+        }
+        function g() {
+            return a;
+        }
+        function h(v) {
+            a = f()[g()] = a = v;
+        }
+        var a = "FAIL";
+        var o = {
+            set FAIL(v) {
+                h(v);
+            }
+        };
+        o.FAIL = "PASS";
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+redundant_assignments: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = a = "PASS", b = "FAIL";
+        b = b = "PASS";
+        console.log(a, b);
+    }
+    expect: {
+        var a = "PASS", b = "FAIL";
+        b = "PASS";
+        console.log(a, b);
+    }
+    expect_stdout: "PASS PASS"
+}
+
+self_assignments_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = "PASS";
+        a = a;
+        console.log(a);
+    }
+    expect: {
+        var a = "PASS";
+        a;
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+self_assignments_2: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "q", o = {
+            p: "PASS",
+        };
+        o.p = o.p;
+        o[a] = o[a];
+        console.log(o.p, o[a]);
+    }
+    expect: {
+        var a = "q", o = {
+            p: "PASS",
+        };
+        console.log(o.p, o[a]);
+    }
+    expect_stdout: "PASS undefined"
+}
+
+self_assignments_3: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "q", o = {
+            p: "FAIL",
+            get q() {
+                return "PASS";
+            },
+            set q(v) {
+                this.p = v;
+            },
+        };
+        o.p = o.p;
+        o[a] = o[a];
+        console.log(o.p, o[a]);
+    }
+    expect: {
+        var a = "q", o = {
+            p: "FAIL",
+            get q() {
+                return "PASS";
+            },
+            set q(v) {
+                this.p = v;
+            },
+        };
+        o.p = o.p;
+        o[a] = o[a];
+        console.log(o.p, o[a]);
+    }
+    expect_stdout: "PASS PASS"
+}
+
+self_assignments_4: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var i = 0, l = [ "PASS" ];
+        l[0] = l[0];
+        l[i] = l[i];
+        console.log(l[0], i);
+    }
+    expect: {
+        var i = 0, l = [ "PASS" ];
+        console.log(l[0], i);
+    }
+    expect_stdout: "PASS 0"
+}
+
+self_assignments_5: {
+    options = {
+        dead_code: true,
+        evaluate: true,
+        passes: 3,
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var i = 0, l = [ "FAIL", "PASS" ];
+        l[0] = l[0];
+        l[i] = l[i];
+        l[i++] = l[i++];
+        console.log(l[0], i);
+    }
+    expect: {
+        var i = 0, l = [ "FAIL", "PASS" ];
+        l[0];
+        l[0];
+        l[0] = l[1];
+        console.log(l[0], 2);
+    }
+    expect_stdout: "PASS 2"
+}
+
+self_assignments_6: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var o = {
+            p: "PASS",
+        };
+        console.log(o.p = o.p);
+    }
+    expect: {
+        var o = {
+            p: "PASS",
+        };
+        console.log(o.p);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3967: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = "FAIL";
+        try {
+            a = 0 in (a = "PASS");
+        } catch (e) {}
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        try {
+            a = 0 in (a = "PASS");
+        } catch (e) {}
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_4051: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        try {
+            delete (A = A);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        try {
+            delete (A = A);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+}
