@@ -699,18 +699,6 @@ iife: {
     }
 }
 
-drop_value: {
-    options = {
-        side_effects: true,
-    }
-    input: {
-        (1, [2, foo()], 3, {a:1, b:bar()});
-    }
-    expect: {
-        foo(), bar();
-    }
-}
-
 issue_1539: {
     options = {
         collapse_vars: true,
@@ -1019,14 +1007,21 @@ delete_assign_1: {
         console.log(delete (a = 0 / 0));
     }
     expect: {
-        console.log((void 0, !0));
-        console.log((void 0, !0));
-        console.log((1 / 0, !0));
-        console.log((1 / 0, !0));
-        console.log((NaN, !0));
-        console.log((NaN, !0));
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
     }
-    expect_stdout: true
+    expect_stdout: [
+        "true",
+        "true",
+        "true",
+        "true",
+        "true",
+        "true",
+    ]
 }
 
 delete_assign_2: {
@@ -1047,14 +1042,21 @@ delete_assign_2: {
         console.log(delete (a = 0 / 0));
     }
     expect: {
-        console.log((void 0, !0));
-        console.log((void 0, !0));
-        console.log((Infinity, !0));
-        console.log((1 / 0, !0));
-        console.log((NaN, !0));
-        console.log((NaN, !0));
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
+        console.log(!0);
     }
-    expect_stdout: true
+    expect_stdout: [
+        "true",
+        "true",
+        "true",
+        "true",
+        "true",
+        "true",
+    ]
 }
 
 drop_var: {
@@ -1189,10 +1191,10 @@ issue_2105_1: {
     input: {
         !function(factory) {
             factory();
-        }( function() {
+        }(function() {
             return function(fn) {
                 fn()().prop();
-            }( function() {
+            }(function() {
                 function bar() {
                     var quux = function() {
                         console.log("PASS");
@@ -1203,7 +1205,7 @@ issue_2105_1: {
                     return { prop: foo };
                 }
                 return bar;
-            } );
+            });
         });
     }
     expect: {
@@ -1233,10 +1235,10 @@ issue_2105_2: {
     input: {
         !function(factory) {
             factory();
-        }( function() {
+        }(function() {
             return function(fn) {
                 fn()().prop();
-            }( function() {
+            }(function() {
                 function bar() {
                     var quux = function() {
                         console.log("PASS");
@@ -1247,7 +1249,7 @@ issue_2105_2: {
                     return { prop: foo };
                 }
                 return bar;
-            } );
+            });
         });
     }
     expect: {
@@ -1256,6 +1258,44 @@ issue_2105_2: {
     expect_stdout: "PASS"
 }
 
+issue_2105_3: {
+    options = {
+        inline: true,
+        passes: 2,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        !function(factory) {
+            factory();
+        }(function() {
+            return function(fn) {
+                fn()().prop();
+            }(function() {
+                function bar() {
+                    var quux = function() {
+                        console.log("PASS");
+                    }, foo = function() {
+                        console.log;
+                        quux();
+                    };
+                    return { prop: foo };
+                }
+                return bar;
+            });
+        });
+    }
+    expect: {
+        !void void {
+            prop: function() {
+                console.log;
+                void console.log("PASS");
+            }
+        }.prop();
+    }
+    expect_stdout: "PASS"
+}
+
 issue_2226_1: {
     options = {
         side_effects: true,
@@ -1635,7 +1675,7 @@ double_assign_2: {
     }
     expect: {
         for (var i = 0; i < 2; i++)
-            void 0, a = {}, console.log(a);
+            a = {}, console.log(a);
         var a;
     }
 }
@@ -1716,7 +1756,7 @@ issue_2768: {
     }
     expect: {
         var a = "FAIL";
-        var c = (d = a, 0, void (d && (a = "PASS")));
+        var c = (d = a, void (d && (a = "PASS")));
         var d;
         console.log(a, typeof c);
     }
@@ -2102,3 +2142,709 @@ issue_3497: {
     }
     expect_stdout: "undefined"
 }
+
+issue_3515_1: {
+    options = {
+        collapse_vars: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var c = 0;
+        (function() {
+            this[c++] = 0;
+            var expr20 = !0;
+            for (var key20 in expr20);
+        })();
+        console.log(c);
+    }
+    expect: {
+        var c = 0;
+        (function() {
+            this[c++] = 0;
+            for (var key20 in !0);
+        })();
+        console.log(c);
+    }
+    expect_stdout: "1"
+}
+
+issue_3515_2: {
+    options = {
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = "FAIL";
+        function f() {
+            typeof b === "number";
+            delete a;
+        }
+        var b = f(a = "PASS");
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        function f() {
+            delete a;
+        }
+        f(a = "PASS");
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3515_3: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        var c = "FAIL";
+        (function() {
+            function f() {
+                c = "PASS";
+            }
+            var a = f();
+            var a = function g(b) {
+                b && (b.p = this);
+            }(a);
+        })();
+        console.log(c);
+    }
+    expect: {
+        var c = "FAIL";
+        (function() {
+            function f() {
+                c = "PASS";
+            }
+            (function(b) {
+                b && (b.p = this);
+            })(f());
+        })();
+        console.log(c);
+    }
+    expect_stdout: "PASS"
+}
+
+function_assign: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            var a = "PASS";
+            function g(b) {
+                return b;
+            }
+            g.p = a;
+            function h(c) {
+                return c;
+            }
+            h.p = a;
+            return h;
+        }().p);
+    }
+    expect: {
+        console.log(function() {
+            var a = "PASS";
+            function h(c) {
+                return c;
+            }
+            h.p = a;
+            return h;
+        }().p);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3598: {
+    options = {
+        collapse_vars: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "FAIL";
+        try {
+            (function() {
+                var b = void 0;
+                a = "PASS";
+                c.p = 0;
+                var c = b[!1];
+            })();
+        } catch (e) {}
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        try {
+            (function() {
+                a = "PASS";
+                (void ((void 0).p = 0))[!1];
+            })();
+        } catch (e) {}
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+}
+
+self_assign: {
+    options = {
+        passes: 2,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        function d(a) {
+            a = a;
+        }
+        function e(a, b) {
+            a = b;
+            b = a;
+        }
+        function f(a, b, c) {
+            a = b;
+            b = c;
+            c = a;
+        }
+        function g(a, b, c) {
+            a = a * b + c;
+        }
+    }
+    expect: {
+        function d(a) {}
+        function e(a, b) {}
+        function f(a, b, c) {}
+        function g(a, b, c) {}
+    }
+}
+
+function_argument_reference: {
+    options = {
+        keep_fargs: false,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = 1, b = 42;
+        function f(a) {
+            b <<= a;
+        }
+        f();
+        console.log(a, b);
+    }
+    expect: {
+        var a = 1, b = 42;
+        function f(a) {
+            b <<= a;
+        }
+        f();
+        console.log(a, b);
+    }
+    expect_stdout: "1 42"
+}
+
+function_parameter_ie8: {
+    options = {
+        ie8: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var a;
+            function f() {
+                console.log("PASS");
+            }
+            f(a = 1 + a);
+        })();
+    }
+    expect: {
+        (function() {
+            (function() {
+                console.log("PASS");
+            })();
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3664: {
+    options = {
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            var a, b = (a = (a = [ b && console.log("FAIL") ]).p = 0, 0);
+            return "PASS";
+        }());
+    }
+    expect: {
+        console.log(function() {
+            var b = ([ b && console.log("FAIL") ].p = 0, 0);
+            return "PASS";
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3673: {
+    options = {
+        pure_getters: "strict",
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        (a = [ a ]).p = 42;
+        console.log("PASS");
+    }
+    expect: {
+        var a;
+        (a = [ a ]).p = 42;
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3746: {
+    options = {
+        keep_fargs: "strict",
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        try {
+            A;
+        } catch (e) {
+            var e;
+        }
+        (function f(a) {
+            e = a;
+        })();
+        console.log("PASS");
+    }
+    expect: {
+        try {
+            A;
+        } catch (e) {
+            var e;
+        }
+        (function(a) {
+            e = a;
+        })();
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+}
+
+drop_duplicated_side_effects: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        for (var i = 1; i--;)
+            var a = 0, b = ++a;
+        console.log(a);
+    }
+    expect: {
+        var a = 0;
+        for (var i = 1; i--;)
+            a = 0, ++a;
+        console.log(a);
+    }
+    expect_stdout: "1"
+}
+
+drop_duplicated_var_catch: {
+    options = {
+        unused: true,
+    }
+    input: {
+        function f() {
+            try {
+                x();
+            } catch (a) {
+                var a, a;
+            }
+        }
+    }
+    expect: {
+        function f() {
+            try {
+                x();
+            } catch (a) {
+                var a;
+            }
+        }
+    }
+}
+
+issue_3802_1: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        a += 0;
+        var a = function() {};
+        console.log(typeof a);
+    }
+    expect: {
+        var a = 0;
+        a += 0;
+        a = function() {};
+        console.log(typeof a);
+    }
+    expect_stdout: "function"
+}
+
+issue_3802_2: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        a += 0;
+        var a = function() {};
+        console.log(typeof a);
+    }
+    expect: {
+        0;
+        var a = function() {};
+        console.log(typeof a);
+    }
+    expect_stdout: "function"
+}
+
+issue_3899: {
+    options = {
+        assignments: true,
+        evaluate: true,
+        functions: true,
+        inline: true,
+        join_vars: true,
+        passes: 2,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        a = a + 1;
+        var a = function f(b) {
+            return function() {
+                return b;
+            };
+        }(2);
+        console.log(typeof a);
+    }
+    expect: {
+        function a() {
+            return 2;
+        }
+        console.log(typeof a);
+    }
+    expect_stdout: "function"
+}
+
+cross_scope_assign_chain: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a, b = 0;
+        (function() {
+            a = b;
+            a++;
+            while (b++);
+        })();
+        console.log(a ? "PASS" : "FAIL");
+    }
+    expect: {
+        var a, b = 0;
+        (function() {
+            a = b;
+            a++;
+            while (b++);
+        })();
+        console.log(a ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+}
+
+assign_if_assign_read: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a) {
+            var b;
+            do {
+                b = "FAIL";
+                if (Array.isArray(a)) {
+                    b = a[0];
+                    console.log(b);
+                }
+            } while (!console);
+        })([ "PASS" ]);
+    }
+    expect: {
+        (function(a) {
+            var b;
+            do {
+                "FAIL";
+                if (Array.isArray(a)) {
+                    b = a[0];
+                    console.log(b);
+                }
+            } while (!console);
+        })([ "PASS" ]);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3951: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = console.log("PASS");
+        console.log(a);
+        a = "0";
+        console.log(a.p = 0);
+        a && a;
+    }
+    expect: {
+        var a = console.log("PASS");
+        console.log(a);
+        a = "0";
+        console.log(a.p = 0);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+        "0",
+    ]
+}
+
+issue_3956: {
+    options = {
+        collapse_vars: true,
+        evaluate: true,
+        inline: true,
+        passes: 2,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        (function(a) {
+            function f(b) {
+                console.log(b);
+                a = 1;
+            }
+            var c = f(c += 0);
+            (function(d) {
+                console.log(d);
+            })(console.log(a) ^ 1, c);
+        })();
+    }
+    expect: {
+        var c, d;
+        c += 0,
+        console.log(NaN),
+        d = 1 ^ console.log(1),
+        console.log(d);
+    }
+    expect_stdout: [
+        "NaN",
+        "1",
+        "1",
+    ]
+}
+
+issue_3962_1: {
+    options = {
+        evaluate: true,
+        keep_fargs: "strict",
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        function f(b, c) {
+            do {
+                var d = console + e, e = 0..toString() === b;
+            } while (0);
+            if (c) console.log("PASS");
+        }
+        var a = f(a--, 1);
+        a;
+    }
+    expect: {
+        var a = 0;
+        a = (function(c) {
+            do {
+                console;
+                0..toString();
+            } while (0);
+            if (c) console.log("PASS");
+        }((a--, 1)), 0);
+        void 0;
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3962_2: {
+    options = {
+        keep_fargs: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        function f(b, c) {
+            do {
+                var d = console + e, e = 0..toString() === b;
+            } while (0);
+            if (c) console.log("PASS");
+        }
+        var a = f(a--, 1);
+        a;
+    }
+    expect: {
+        var a = 0;
+        a = (function(c) {
+            do {
+                console;
+                0..toString();
+            } while (0);
+            if (c) console.log("PASS");
+        }((a--, 1)), 0);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3986: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0, b = 0;
+        (function() {
+            try {
+                throw 42;
+            } catch (e) {
+                a++;
+            }
+            b = b && 0;
+        })(b *= a);
+        console.log(b);
+    }
+    expect: {
+        var a = 0, b = 0;
+        (function() {
+            try {
+                throw 42;
+            } catch (e) {
+                a++;
+            }
+            b = b && 0;
+        })(b *= a);
+        console.log(b);
+    }
+    expect_stdout: "0"
+}
+
+issue_4017: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        console.log(function f() {
+            var b = c &= 0;
+            var c = a++ + (A = a);
+            var d = c && c[f];
+        }());
+    }
+    expect: {
+        var a = 0;
+        console.log(function() {
+            c &= 0;
+            var c = (a++, A = a, 0);
+        }());
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4025: {
+    options = {
+        collapse_vars: true,
+        evaluate: true,
+        passes: 2,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0, b = 0, c = 0, d = a++;
+        try {
+            var e = console.log(c), f = b;
+        } finally {
+            var d = b = 1, d = c + 1;
+            c = 0;
+        }
+        console.log(a, b, d);
+    }
+    expect: {
+        var d, c = 0;
+        try {
+            console.log(c);
+        } finally {
+            d = c + 1;
+            c = 0;
+        }
+        console.log(1, 1, d);
+    }
+    expect_stdout: [
+        "0",
+        "1 1 1",
+    ]
+}
