@@ -237,22 +237,39 @@ unsafe_constant: {
         unsafe: true,
     }
     input: {
-        console.log(
-            true.a,
-            false.a,
-            null.a,
-            undefined.a
-        );
+        console.log(true.a, false.a);
+        console.log(true.valueOf(), false.valueOf());
+        try {
+            console.log(null.a);
+        } catch (e) {
+            console.log("PASS");
+        }
+        try {
+            console.log(undefined.a);
+        } catch (e) {
+            console.log("PASS");
+        }
     }
     expect: {
-        console.log(
-            void 0,
-            false.a,
-            null.a,
-            (void 0).a
-        );
+        console.log(void 0, void 0);
+        console.log(true, false);
+        try {
+            console.log(null.a);
+        } catch (e) {
+            console.log("PASS");
+        }
+        try {
+            console.log((void 0).a);
+        } catch (e) {
+            console.log("PASS");
+        }
     }
-    expect_stdout: true
+    expect_stdout: [
+        "undefined undefined",
+        "true false",
+        "PASS",
+        "PASS",
+    ]
 }
 
 unsafe_object: {
@@ -541,34 +558,51 @@ unsafe_array: {
         unsafe: true,
     }
     input: {
-        console.log(
-            [1, , 3][1],
-            [1, 2, 3, a] + 1,
-            [1, 2, 3, 4] + 1,
-            [1, 2, 3, a][0] + 1,
-            [1, 2, 3, 4][0] + 1,
-            [1, 2, 3, 4][6 - 5] + 1,
-            [1, , 3, 4][6 - 5] + 1,
-            [[1, 2], [3, 4]][0] + 1,
-            [[1, 2], [3, 4]][6 - 5][1] + 1,
-            [[1, 2], , [3, 4]][6 - 5][1] + 1
-        );
+        var a = "PASS";
+        Array.prototype[1] = a;
+        console.log([, ].length);
+        console.log("" + [, , ]);
+        console.log([1, , 3][1]);
+        console.log([1, 2, 3, a] + 1);
+        console.log([1, 2, 3, 4] + 1);
+        console.log([1, 2, 3, a][0] + 1);
+        console.log([1, 2, 3, 4][0] + 1);
+        console.log([1, 2, 3, 4][6 - 5] + 1);
+        console.log([1, , 3, 4][6 - 5] + 1);
+        console.log([[1, 2], [3, 4]][0] + 1);
+        console.log([[1, 2], [3, 4]][6 - 5][1] + 1);
+        console.log([[1, 2], , [3, 4]][6 - 5][1] + 1);
     }
     expect: {
-        console.log(
-            void 0,
-            [1, 2, 3, a] + 1,
-            "1,2,3,41",
-            [1, 2, 3, a][0] + 1,
-            2,
-            3,
-            NaN,
-            "1,21",
-            5,
-            (void 0)[1] + 1
-        );
+        var a = "PASS";
+        Array.prototype[1] = a;
+        console.log([, ].length);
+        console.log("" + [, , ]);
+        console.log([1, , 3][1]);
+        console.log([1, 2, 3, a] + 1);
+        console.log("1,2,3,41");
+        console.log([1, 2, 3, a][0] + 1);
+        console.log(2);
+        console.log(3);
+        console.log([1, , 3, 4][1] + 1);
+        console.log("1,21");
+        console.log(5);
+        console.log([[1, 2], , [3, 4]][1][1] + 1);
     }
-    expect_stdout: true
+    expect_stdout: [
+        "1",
+        ",PASS",
+        "PASS",
+        "1,2,3,PASS1",
+        "1,2,3,41",
+        "2",
+        "2",
+        "3",
+        "PASS1",
+        "1,21",
+        "5",
+        "A1",
+    ]
 }
 
 unsafe_string: {
@@ -1562,9 +1596,9 @@ issue_2968_1: {
     expect: {
         var c = "FAIL";
         (function() {
-            b = -(a = 42),
-            void ((a <<= 0) && (a[(c = "PASS", 0 >>> (b += 1))] = 0));
-            var a, b;
+            a = 42,
+            void ((a <<= 0) && (a[(c = "PASS", 0)] = 0));
+            var a;
         })();
         console.log(c);
     }
@@ -1757,3 +1791,1081 @@ issue_3387_2: {
     }
     expect_stdout: "NaN"
 }
+
+simple_function_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function sum(a, b) {
+            return a + b;
+        }
+        console.log(sum(1, 2) * sum(3, 4));
+    }
+    expect: {
+        console.log(21);
+    }
+    expect_stdout: "21"
+}
+
+simple_function_2: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var sum = function(a, b) {
+            return a + b;
+        }
+        console.log(sum(1, 2) * sum(3, 4));
+    }
+    expect: {
+        console.log(21);
+    }
+    expect_stdout: "21"
+}
+
+recursive_function_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function factorial(a) {
+            return a > 0 ? a * factorial(a - 1) : 1;
+        }
+        console.log(factorial(5));
+    }
+    expect: {
+        console.log(function factorial(a) {
+            return a > 0 ? a * factorial(a - 1) : 1;
+        }(5));
+    }
+    expect_stdout: "120"
+}
+
+recursive_function_2: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var factorial = function(a) {
+            return a > 0 ? a * factorial(a - 1) : 1;
+        }
+        console.log(factorial(5));
+    }
+    expect: {
+        var factorial = function(a) {
+            return a > 0 ? a * factorial(a - 1) : 1;
+        }
+        console.log(factorial(5));
+    }
+    expect_stdout: "120"
+}
+
+issue_3558: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        function f(a) {
+            return 1 + --a;
+        }
+        console.log(f(true), f(false));
+    }
+    expect: {
+        function f(a) {
+            return 1 + --a;
+        }
+        console.log(1, 0);
+    }
+    expect_stdout: "1 0"
+}
+
+issue_3568: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unsafe: true,
+    }
+    input: {
+        var a = 0;
+        function f(b) {
+            return b && b.p;
+        }
+        console.log(f(++a + f()));
+    }
+    expect: {
+        var a = 0;
+        function f(b) {
+            return b && b.p;
+        }
+        console.log(NaN);
+    }
+    expect_stdout: "NaN"
+}
+
+conditional_function: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        function f(a) {
+            return a && "undefined" != typeof A ? A : 42;
+        }
+        console.log(f(0), f(1));
+    }
+    expect: {
+        function f(a) {
+            return a && "undefined" != typeof A ? A : 42;
+        }
+        console.log(42, f(1));
+    }
+    expect_stdout: "42 42"
+}
+
+best_of_evaluate: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function d(x, y) {
+            return x / y;
+        }
+        console.log(0 / 3, 1 / 64, 4 / 7, 7 / 7);
+        console.log(d(0, 3), d(1, 64), d(4, 7), d(7, 7));
+    }
+    expect: {
+        function d(x, y) {
+            return x / y;
+        }
+        console.log(0, 1 / 64, 4 / 7, 1);
+        console.log(0, .015625, d(4, 7), 1);
+    }
+    expect_stdout: [
+        "0 0.015625 0.5714285714285714 1",
+        "0 0.015625 0.5714285714285714 1",
+    ]
+}
+
+eager_evaluate: {
+    options = {
+        evaluate: "eager",
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function d(x, y) {
+            return x / y;
+        }
+        console.log(0 / 3, 1 / 64, 4 / 7, 7 / 7);
+        console.log(d(0, 3), d(1, 64), d(4, 7), d(7, 7));
+    }
+    expect: {
+        console.log(0, .015625, .5714285714285714, 1);
+        console.log(0, .015625, .5714285714285714, 1);
+    }
+    expect_stdout: [
+        "0 0.015625 0.5714285714285714 1",
+        "0 0.015625 0.5714285714285714 1",
+    ]
+}
+
+threshold_evaluate_default: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log(b("1"), b(2), b(b(b("ABCDEFGHIJK"))));
+    }
+    expect: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log("111", 6, b(b(b("ABCDEFGHIJK"))));
+    }
+    expect_stdout: "111 6 ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK"
+}
+
+threshold_evaluate_30: {
+    options = {
+        evaluate: 30,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log(b("1"), b(2), b(b(b("ABCDEFGHIJK"))));
+    }
+    expect: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log("111", 6, b(b("ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK")));
+    }
+    expect_stdout: "111 6 ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK"
+}
+
+threshold_evaluate_100: {
+    options = {
+        evaluate: 100,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log(b("1"), b(2), b(b(b("ABCDEFGHIJK"))));
+    }
+    expect: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log("111", 6, b("ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK"));
+    }
+    expect_stdout: "111 6 ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK"
+}
+
+threshold_evaluate_999: {
+    options = {
+        evaluate: 999,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function b(x) {
+            return x + x + x;
+        }
+        console.log(b("1"), b(2), b(b(b("ABCDEFGHIJK"))));
+    }
+    expect: {
+        console.log("111", 6, "ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK");
+    }
+    expect_stdout: "111 6 ABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJKABCDEFGHIJK"
+}
+
+collapse_vars_regexp: {
+    options = {
+        booleans: true,
+        collapse_vars: true,
+        comparisons: true,
+        conditionals: true,
+        dead_code: true,
+        evaluate: true,
+        hoist_funs: true,
+        if_return: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: false,
+        reduce_funcs: true,
+        reduce_vars: true,
+        side_effects: true,
+        unsafe_regexp: true,
+        unused: true,
+    }
+    input: {
+        function f1() {
+            var k = 9;
+            var rx = /[A-Z]+/;
+            return [rx, k];
+        }
+        function f2() {
+            var rx = /ab*/g;
+            return function(s) {
+                return rx.exec(s);
+            };
+        }
+        function f3() {
+            var rx = /ab*/g;
+            return function() {
+                return rx;
+            };
+        }
+        (function() {
+            var result;
+            var s = "acdabcdeabbb";
+            var rx = /ab*/g;
+            while (result = rx.exec(s))
+                console.log(result[0]);
+        })();
+        (function() {
+            var result;
+            var s = "acdabcdeabbb";
+            var rx = f2();
+            while (result = rx(s))
+                console.log(result[0]);
+        })();
+        (function() {
+            var result;
+            var s = "acdabcdeabbb";
+            var rx = f3();
+            while (result = rx().exec(s))
+                console.log(result[0]);
+        })();
+    }
+    expect: {
+        function f1() {
+            return [/[A-Z]+/, 9];
+        }
+        function f2() {
+            var rx = /ab*/g;
+            return function(s) {
+                return rx.exec(s);
+            };
+        }
+        function f3() {
+            var rx = /ab*/g;
+            return function() {
+                return rx;
+            };
+        }
+        (function() {
+            var result, rx = /ab*/g;
+            while (result = rx.exec("acdabcdeabbb"))
+                console.log(result[0]);
+        })();
+        (function() {
+            var result, rx = f2();
+            while (result = rx("acdabcdeabbb"))
+                console.log(result[0]);
+        })();
+        (function() {
+            var result, rx = f3();
+            while (result = rx().exec("acdabcdeabbb"))
+                console.log(result[0]);
+        })();
+    }
+    expect_stdout: [
+        "a",
+        "ab",
+        "abbb",
+        "a",
+        "ab",
+        "abbb",
+        "a",
+        "ab",
+        "abbb",
+    ]
+}
+
+issue_3738: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log(1 / (0 + ([] - 1) % 1));
+    }
+    expect: {
+        console.log(1 / (0 + ([] - 1) % 1));
+    }
+    expect_stdout: "Infinity"
+}
+
+issue_3755: {
+    options = {
+        booleans: true,
+        evaluate: true,
+        unsafe: true,
+        unsafe_math: true,
+    }
+    input: {
+        console.log((/4/.exec(1 + (!0 - 5 / "23")) || 0).p);
+    }
+    expect: {
+        console.log((/4/.exec(!0 - 5 / "23" + 1), 0).p);
+    }
+    expect_stdout: "undefined"
+}
+
+void_side_effects: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = void console.log("PASS");
+        console.log(a);
+    }
+    expect: {
+        console.log("PASS");
+        console.log(void 0);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+    ]
+}
+
+no_returns: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function() {
+            console.log("PASS");
+        }();
+        console.log(a);
+    }
+    expect: {
+        (function() {
+            console.log("PASS");
+        })();
+        console.log(void 0);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+    ]
+}
+
+void_returns: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function f() {
+            function g(b) {
+                if (b) console.log("FAIL");
+            }
+            while (1) {
+                console.log("PASS");
+                try {
+                    if (console) return;
+                } catch (e) {
+                    return g(e);
+                }
+            }
+        }();
+        console.log(a);
+    }
+    expect: {
+        (function() {
+            function g(b) {
+                if (b) console.log("FAIL");
+            }
+            while (1) {
+                console.log("PASS");
+                try {
+                    if (console) return;
+                } catch (e) {
+                    return g(e);
+                }
+            }
+        })();
+        console.log(void 0);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+    ]
+}
+
+void_returns_recursive: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function f() {
+            function g(b) {
+                return f();
+            }
+            while (1) {
+                console.log("PASS");
+                try {
+                    if (console) return;
+                } catch (e) {
+                    return g(e);
+                }
+            }
+        }();
+        console.log(a);
+    }
+    expect: {
+        var a = function f() {
+            function g(b) {
+                return f();
+            }
+            while (1) {
+                console.log("PASS");
+                try {
+                    if (console) return;
+                } catch (e) {
+                    return g();
+                }
+            }
+        }();
+        console.log(a);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+    ]
+}
+
+issue_3878_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var b = function(a) {
+            return (a = 0) == (a && this > (a += 0));
+        }();
+        console.log(b ? "PASS" : "FAIL");
+    }
+    expect: {
+        console.log(true ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3878_2: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "foo";
+        a++ + a;
+        a && a;
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        a++ + a;
+        a;
+        console.log(a);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_3882: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f(a) {
+            return console.log(a++), a && this;
+        }
+        var b = f();
+        console.log(b);
+    }
+    expect: {
+        var b = function(a) {
+            return console.log(a++), a && this;
+        }();
+        console.log(b);
+    }
+    expect_stdout: [
+        "NaN",
+        "NaN",
+    ]
+}
+
+issue_3887: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(b) {
+            try {
+                b-- && console.log("PASS");
+            } catch (a_2) {}
+        })(1);
+    }
+    expect: {
+        (function(b) {
+            try {
+                1, console.log("PASS");
+            } catch (a_2) {}
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3903: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS";
+        function f(b, c) {
+            return console, c;
+        }
+        var d = f(f(), a = a);
+        console.log(d);
+    }
+    expect: {
+        function f(b, c) {
+            return console, c;
+        }
+        f(f(), "PASS");
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3905: {
+    options = {
+        evaluate: true,
+        passes: 2,
+        unused: true,
+    }
+    input: {
+        (function(a, a) {
+            return console.log(a = 0), a && console.log("FAIL");
+        })("foo", 42);
+    }
+    expect: {
+        (function(a, a) {
+            return console.log(a = 0), a && console.log("FAIL");
+        })("foo", 42);
+    }
+    expect_stdout: "0"
+}
+
+issue_3920: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function(b) {
+            return (b[b = 0] = 0) >= (b ? 0 : 1);
+        }("foo");
+        console.log(a);
+    }
+    expect: {
+        (function(b) {
+            "foo"[0] = 0;
+        })();
+        console.log(false);
+    }
+    expect_stdout: "false"
+}
+
+inlined_increment_prefix: {
+    options = {
+        evaluate: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 0;
+        (function() {
+            ++a;
+        })();
+        console.log(a += 0);
+    }
+    expect: {
+        var a = 0;
+        void ++a;
+        console.log(1);
+    }
+    expect_stdout: "1"
+}
+
+inlined_increment_postfix: {
+    options = {
+        evaluate: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 0;
+        (function() {
+            a++;
+        })();
+        console.log(a += 0);
+    }
+    expect: {
+        var a = 0;
+        void a++;
+        console.log(1);
+    }
+    expect_stdout: "1"
+}
+
+compound_assignment_to_property: {
+    options = {
+        evaluate: true,
+        unsafe: true,
+    }
+    input: {
+        1 + (0..p >>= 0) && console.log("PASS");
+    }
+    expect: {
+        1 + (0..p >>= 0),
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_2208_assign: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        a = 42;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect: {
+        a = 42;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect_stdout: "42"
+}
+
+issue_2208_postfix: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        a = 41;
+        a++;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect: {
+        a = 41;
+        a++;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect_stdout: "42"
+}
+
+issue_2208_prefix: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        a = 43;
+        --a;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect: {
+        a = 43;
+        --a;
+        console.log({
+            p: function() {
+                return function() {
+                    return this.a;
+                }();
+            }
+        }.p());
+    }
+    expect_stdout: "42"
+}
+
+issue_3933: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, b) {
+            a && (b ^= 1) && console.log("PASS");
+        })(1);
+    }
+    expect: {
+        (function(a, b) {
+            1, 1, console.log("PASS");
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3935: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log(function f(a) {
+            return a++;
+        }());
+    }
+    expect: {
+        console.log(NaN);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_3937: {
+    options = {
+        conditionals: true,
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unsafe: true,
+    }
+    input: {
+        var a = 123;
+        (a++ + (b = a))[b] ? 0 ? a : b : 0 ? a : b;
+        console.log(a, b);
+    }
+    expect: {
+        var a = 123;
+        (a++ + (b = a))[b], 0, b;
+        console.log(a, b);
+    }
+    expect_stdout: "124 124"
+}
+
+issue_3944: {
+    options = {
+        collapse_vars: true,
+        evaluate: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            function f() {
+                while (function() {
+                    var a = 0 == (b && b.p), b = console.log(a);
+                }());
+                f;
+            }
+            f();
+        })();
+    }
+    expect: {
+        void function f() {
+            while (a = 0 == (a = void 0), console.log(a), void 0);
+            var a;
+            f;
+        }();
+    }
+    expect_stdout: "false"
+}
+
+issue_3953: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+    }
+    input: {
+        function f(a) {
+            (a += 0 * (a = 0)) && console.log("PASS");
+        }
+        f(1);
+    }
+    expect: {
+        function f(a) {
+            (a += 0 * (a = 0)) && console.log("PASS");
+        }
+        f(1);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3988: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unsafe: true,
+        unused: true,
+    }
+    input: {
+        function f(b) {
+            return ("" + (b &= 0))[b && this];
+        }
+        var a = f();
+        console.log(a);
+    }
+    expect: {
+        var a = function(b) {
+            return ("" + (b &= 0))[b && this];
+        }();
+        console.log(a);
+    }
+    expect_stdout: "0"
+}
+
+operator_in: {
+    options = {
+        evaluate: true,
+        unsafe: true,
+    }
+    input: {
+        Object.prototype.PASS = 0;
+        console.log(0 in [ 1 ]);
+        console.log(0 in [ , ]);
+        console.log(0 / 0 in { NaN: 2 });
+        console.log("PASS" in { });
+        console.log("FAIL" in { });
+        console.log("toString" in { });
+        console.log("toString" in { toString: 3 });
+    }
+    expect: {
+        Object.prototype.PASS = 0;
+        console.log(true);
+        console.log(0 in [ , ]);
+        console.log(true);
+        console.log("PASS" in { });
+        console.log("FAIL" in { });
+        console.log("toString" in { });
+        console.log(true);
+    }
+    expect_stdout: [
+        "true",
+        "false",
+        "true",
+        "true",
+        "false",
+        "true",
+        "true",
+    ]
+}
+
+issue_3997: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = function f(b) {
+            return b[b += this] = b;
+        }(0);
+        console.log(typeof a);
+    }
+    expect: {
+        var a = function f(b) {
+            return b[b += this] = b;
+        }(0);
+        console.log(typeof a);
+    }
+    expect_stdout: "string"
+}
+
+issue_4035: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+    }
+    input: {
+        var a = 0;
+        (function() {
+            var b = --a;
+            console.log(delete (0 + b));
+            console.log(delete (1 * b));
+            console.log(delete (b + 0));
+            console.log(delete (b - 0));
+            console.log(delete (b / 1));
+        })();
+    }
+    expect: {
+        var a = 0;
+        (function() {
+            var b = --a;
+            console.log((0 + b, true));
+            console.log((1 * b, true));
+            console.log((0 + b, true));
+            console.log((b - 0, true));
+            console.log((b / 1, true));
+        })();
+    }
+    expect_stdout: [
+        "true",
+        "true",
+        "true",
+        "true",
+        "true",
+    ]
+}
