@@ -664,7 +664,7 @@ issue_2519: {
     }
     expect: {
         function testFunc() {
-            return 1 * ((6 + 5) / 2);
+            return +((6 + 5) / 2);
         }
         console.log(testFunc());
     }
@@ -767,18 +767,17 @@ issue_3071_1: {
             var obj = {};
             obj.one = 1;
             obj.two = 2;
-            console.log(obj.one);
+            console.log(obj.one, obj.two);
         })();
     }
     expect: {
-        console.log(1);
+        console.log(1, 2);
     }
-    expect_stdout: "1"
+    expect_stdout: "1 2"
 }
 
 issue_3071_2: {
     options = {
-        evaluate: true,
         hoist_props: true,
         inline: true,
         join_vars: true,
@@ -793,19 +792,18 @@ issue_3071_2: {
             obj = {};
             obj.one = 1;
             obj.two = 2;
-            console.log(obj.one);
+            console.log(obj.one, obj.two);
             var obj;
         })();
     }
     expect: {
-        console.log(1);
+        console.log(1, 2);
     }
-    expect_stdout: "1"
+    expect_stdout: "1 2"
 }
 
 issue_3071_2_toplevel: {
     options = {
-        evaluate: true,
         hoist_props: true,
         inline: true,
         join_vars: true,
@@ -821,14 +819,14 @@ issue_3071_2_toplevel: {
             obj = {};
             obj.one = 1;
             obj.two = 2;
-            console.log(obj.one);
+            console.log(obj.one, obj.two);
             var obj;
         })();
     }
     expect: {
-        console.log(1);
+        console.log(1, 2);
     }
-    expect_stdout: "1"
+    expect_stdout: "1 2"
 }
 
 issue_3071_3: {
@@ -914,3 +912,132 @@ issue_3440: {
     }
     expect_stdout: "PASS"
 }
+
+issue_3868: {
+    options = {
+        hoist_props: true,
+        passes: 2,
+        reduce_vars: true,
+        side_effects: true,
+    }
+    input: {
+        (function(t) {
+            t = {};
+            ({
+                get p() {},
+                q: (console.log("PASS"), +t),
+            }).r;
+        })();
+    }
+    expect: {
+        (function(t) {
+            t = {};
+            ({
+                get p() {},
+                q: (console.log("PASS"), +t),
+            }).r;
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3871: {
+    options = {
+        hoist_props: true,
+        reduce_vars: true,
+    }
+    input: {
+        console.log(function() {
+            do {
+                var b = {
+                    get null() {
+                        c;
+                    }
+                };
+            } while (!b);
+            return "PASS";
+        }());
+    }
+    expect: {
+        console.log(function() {
+            do {
+                var b = {
+                    get null() {
+                        c;
+                    }
+                };
+            } while (!b);
+            return "PASS";
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3945_1: {
+    options = {
+        hoist_props: true,
+        reduce_vars: true,
+    }
+    input: {
+        function f() {
+            o.p;
+            var o = {
+                q: 0,
+            };
+        }
+    }
+    expect: {
+        function f() {
+            o.p;
+            var o = {
+                q: 0,
+            };
+        }
+    }
+}
+
+issue_3945_2: {
+    options = {
+        hoist_props: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        console.log(typeof o);
+        var o = {
+            p: 0,
+        };
+    }
+    expect: {
+        console.log(typeof o);
+        var o = {
+            p: 0,
+        };
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4023: {
+    options = {
+        comparisons: true,
+        hoist_props: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+        typeofs: true,
+        unused: true,
+    }
+    input: {
+        function f() {
+            var a = function() {
+                return { p: 0 };
+            }();
+            return console.log("undefined" != typeof a);
+        }
+        f();
+    }
+    expect: {
+        console.log(void 0 !== {});
+    }
+    expect_stdout: "true"
+}
